{% include 'project.cmake.j2' %}

stm32_get_chip_info($ENV{STM32_CHIP} FAMILY STM32_FAMILY DEVICE STM32_DEVICE TYPE STM32_TYPE)

message("Fetching stm32 cube for stm32 ${STM32_FAMILY} family")
stm32_fetch_cube(${STM32_FAMILY})
