/**
 * @copyright Copyright (C) 2021 Spyrosoft Solutions. All rights reserved.
 */
#pragma once

{% for include in header.includes %}
#include "{{include.name}}"
{% endfor %}

{% for ns in header.namespaces %}
namespace {{ns.name}} {
{% endfor %}

{% for cls in header.classes %}
{% if cls.name %}class {{cls.name}} {
public:
{% endif %}
{% for method in cls.methods %}
{% if cls.name %}    {% endif %}{{method.return_type}} {{method.name}}({{method.get_arg_types()}});
{% endfor %}
{% if cls.name %}
};
{% endif %}
{% endfor %}

{% for ns in header.namespaces %}
} // {{ns.name}}
{% endfor %}
