//
// Copyright
//

#pragma once

/**
 * @brief This is example function which prints hello in std output
 *
 */
namespace spyro
{
class {{ class_name }}
{
public:
    {{ class_name }}();
};
}  // namespace spyro
